@@ -5,16 +5,16 @@
 #include "autoopts/options.h"
 #include "autoopts/usage-txt.h"
 #include "genshell.h"
-#include "xat-attribute.h"
-#include "value-type.h"
+#include "option-xat-attribute.h"
+#include "option-value-type.h"
 #include "ao-strs.h"
 #include "ag-char-map.h"
 #include "autoopts.h"
 #include "proto.h"
 #include "parse-duration.c"
 #include "ao-strs.c"
-#include "value-type.c"
-#include "xat-attribute.c"
+#include "option-value-type.c"
+#include "option-xat-attribute.c"
 #include "autoopts.c"
 #include "alias.c"
 #include "boolean.c"
@@ -43,3 +43,4 @@
 #include "tokenize.c"
 #include "usage.c"
 #include "version.c"
+#include "init.c"
